@@ -64,10 +64,11 @@ def main():
     # perform https setup
     if args.use_https:
         https_setup(args, get_pilot_version())
+    args.amq = None
 
     # let the server know that the worker has started
     if args.update_server:
-        send_worker_status('started', args.queue, args.url, args.port, logger)
+        send_worker_status('started', args.queue, args.url, args.port, logger, 'IPv6')  # note: assuming IPv6, fallback in place
 
     # initialize InfoService
     try:
@@ -80,8 +81,14 @@ def main():
         logger.fatal(error)
         return error.get_error_code()
 
-    # set the site name for rucio  ## is it really used?
-    environ['PILOT_RUCIO_SITENAME'] = infosys.queuedata.site
+    # handle special CRIC variables via params
+    # internet protocol versions 'IPv4' or 'IPv6' can be set via CRIC PQ.params.internet_protocol_version
+    # (must be defined per PQ if wanted). The pilot default is IPv6
+    args.internet_protocol_version = infosys.queuedata.params.get('internet_protocol_version', 'IPv6') if infosys.queuedata.params else 'IPv6'
+    environ['PILOT_IP_VERSION'] = args.internet_protocol_version
+
+    # set the site name for rucio
+    environ['PILOT_RUCIO_SITENAME'] = os.environ.get('PILOT_RUCIO_SITENAME', None) or infosys.queuedata.site
 
     # store the site name as set with a pilot option
     environ['PILOT_SITENAME'] = infosys.queuedata.resource  #args.site  # TODO: replace with singleton
@@ -99,7 +106,7 @@ def main():
 
     # let the server know that the worker has finished
     if args.update_server:
-        send_worker_status('finished', args.queue, args.url, args.port, logger)
+        send_worker_status('finished', args.queue, args.url, args.port, logger, args.internet_protocol_version)
 
     return exitcode
 
@@ -190,7 +197,7 @@ def get_args():
                             default='PR',
                             help='Version tag (default: PR, optional: RC)')
 
-    arg_parser.add_argument('-z', '--updateserver',
+    arg_parser.add_argument('-z', '--noserverupdate',
                             dest='update_server',
                             action='store_false',
                             default=True,
@@ -223,6 +230,13 @@ def get_args():
                             type=int,
                             help='Maximum number of getjob request failures in Harvester mode')
 
+    arg_parser.add_argument('--subscribe-to-msgsvc',
+                            dest='subscribe_to_msgsvc',
+                            action='store_true',
+                            default=False,
+                            required=False,
+                            help='Ask Pilot to receive job/task info from ActiveMQ')
+
     # SSL certificates
     arg_parser.add_argument('--cacert',
                             dest='cacert',
@@ -256,6 +270,10 @@ def get_args():
                             dest='rucio_host',
                             default='',
                             help='URL for the Rucio host (optional)')
+    arg_parser.add_argument('--redirect-stdout',
+                            dest='redirectstdout',
+                            default='',
+                            help='Redirect all stdout to given file, or /dev/null (optional)')
 
     # Country group
     arg_parser.add_argument('--country-group',
@@ -545,7 +563,7 @@ def get_pilot_source_dir():
         return cwd
 
 
-def send_worker_status(status, queue, url, port, logger):
+def send_worker_status(status, queue, url, port, logger, internet_protocol_version):
     """
     Send worker info to the server to let it know that the worker has started
     Note: the function can fail, but if it does, it will be ignored.
@@ -555,6 +573,7 @@ def send_worker_status(status, queue, url, port, logger):
     :param url: server url (string).
     :param port: server port (string).
     :param logger: logging object.
+    :param internet_protocol_version: internet protocol version, IPv4 or IPv6 (string).
     :return:
     """
 
@@ -567,11 +586,44 @@ def send_worker_status(status, queue, url, port, logger):
 
     # attempt to send the worker info to the server
     if data['workerID'] and data['harvesterID']:
-        send_update('updateWorkerPilotStatus', data, url, port)
+        send_update('updateWorkerPilotStatus', data, url, port, ipv=internet_protocol_version)
     else:
         logger.warning('workerID/harvesterID not known, will not send worker status to server')
 
 
+def set_lifetime(args):
+    """
+    Update the pilot lifetime if set by an environment variable (PANDAPILOT_LIFETIME) (in seconds).
+
+    :param args: pilot args.
+    :return:
+    """
+
+    lifetime = os.environ.get('PANDAPILOT_LIFETIME', None)
+    if lifetime:
+        try:
+            lifetime = int(lifetime)
+        except (ValueError, TypeError):
+            pass
+        else:
+            args.lifetime = lifetime
+
+
+def set_redirectall(args):
+    """
+
+    """
+
+    redirectall = os.environ.get('PANDAPILOT_REDIRECTALL', False)
+    if not redirectall:
+        try:
+            redirectall = bool(redirectall)
+        except (ValueError, TypeError):
+            pass
+        else:
+            args.redirectall = redirectall
+
+
 if __name__ == '__main__':
     """
     Main function of pilot module.
@@ -591,6 +643,11 @@ if __name__ == '__main__':
     # initialize job status dictionary (e.g. used to keep track of log transfers)
     args.job_status = {}  # TODO: move to singleton or to job object directly?
 
+    #if 'BNL_OSG_SPHENIX_TEST' in args.queue:
+    #    args.lifetime = 3600
+    #    args.subscribe_to_msgsvc = True
+    #    args.redirectstdout = '/dev/null'
+
     # store T0 time stamp
     add_to_pilot_timing('0', PILOT_START_TIME, time.time(), args)
     add_to_pilot_timing('1', PILOT_MULTIJOB_START_TIME, time.time(), args)
@@ -602,8 +659,10 @@ if __name__ == '__main__':
     if exit_code != 0:
         sys.exit(exit_code)
 
+    set_lifetime(args)
+
     # setup and establish standard logging
-    establish_logging(debug=args.debug, nopilotlog=args.nopilotlog)
+    establish_logging(debug=args.debug, nopilotlog=args.nopilotlog, redirectstdout=args.redirectstdout)
 
     # set environment variables (to be replaced with singleton implementation)
     set_environment_variables()
