@@ -91,7 +91,7 @@ def control(queues, traces, args):
     else:
         logger.debug('will not set job_aborted yet')
 
-    logger.debug('[payload] control thread has finished')
+    logger.info('[payload] control thread has finished')
 
 
 def validate_pre(queues, traces, args):
@@ -259,16 +259,16 @@ def execute_payloads(queues, traces, args):  # noqa: C901
             perform_initial_payload_error_analysis(job, exit_code)
 
             # was an error already found?
-            #if job.piloterrorcodes:
-            #    exit_code_interpret = 1
-            #else:
             user = __import__('pilot.user.%s.diagnose' % pilot_user, globals(), locals(), [pilot_user], 0)
             try:
                 exit_code_interpret = user.interpret(job)
             except Exception as error:
                 logger.warning(f'exception caught: {error}')
-                #exit_code_interpret = -1
-                job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(errors.INTERNALPILOTPROBLEM)
+                if 'error code:' in error and 'message:' in error:
+                    error_code, diagnostics = extract_error_info(error)
+                    job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(error_code, msg=diagnostics)
+                else:
+                    job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(errors.INTERNALPILOTPROBLEM, msg=error)
 
             if job.piloterrorcodes:
                 exit_code_interpret = 1
@@ -292,7 +292,6 @@ def execute_payloads(queues, traces, args):  # noqa: C901
             logger.fatal(f'execute payloads caught an exception (cannot recover): {error}, {traceback.format_exc()}')
             if job:
                 job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(errors.PAYLOADEXECUTIONEXCEPTION)
-                #queues.failed_payloads.put(job)
                 put_in_queue(job, queues.failed_payloads)
             while not args.graceful_stop.is_set():
                 # let stage-out of log finish, but stop running payloads as there should be a problem with the pilot
@@ -308,6 +307,29 @@ def execute_payloads(queues, traces, args):  # noqa: C901
     logger.info('[payload] execute_payloads thread has finished')
 
 
+def extract_error_info(error):
+    """
+    Extract the error code and diagnostics from an error exception.
+
+    :param error: exception string.
+    :return: error code (int), diagnostics (string).
+    """
+
+    error_code = errors.INTERNALPILOTPROBLEM
+    diagnostics = f'full exception: {error}'
+
+    pattern = r'error\ code\:\ ([0-9]+)\,\ message\:\ (.+)'
+    errinfo = findall(pattern, error)
+    if errinfo:  # e.g. [('1303', 'Failed during file handling')]
+        try:
+            error_code = errinfo[0][0]
+            diagnostics = errinfo[0][1]
+        except Exception as exc:
+            logger.warning(f'failed to extract error info from exception error={error}, exc={exc}')
+
+    return error_code, diagnostics
+
+
 def get_transport(catchall):
     """
     Extract the transport/protocol from catchall if present.
@@ -339,6 +361,7 @@ def get_logging_info(job, args):
     info_dic = {}
 
     if not job.realtimelogging:
+        logger.info("job.realtimelogging is not enabled")
         return {}
 
     # args handling
@@ -352,21 +375,7 @@ def get_logging_info(job, args):
         logger.warning('not enough info available for activating real-time logging')
         return {}
 
-    if info:
-        try:
-            info_dic['logging_type'] = info[0][0]
-            info_dic['protocol'] = info[0][1]
-            info_dic['url'] = info[0][2]
-            info_dic['port'] = info[0][3]
-        except IndexError as exc:
-            print(f'exception caught: {exc}')
-            info_dic = {}
-        else:
-            # find the log file to tail
-            path = find_log_to_tail(job.debug_command, job.workdir, args, job.is_analysis())
-            logger.info(f'using {path} for real-time logging')
-            info_dic['logfiles'] = [path]
-    else:
+    if len(logserver) > 0:
         items = logserver.split(':')
         info_dic['logging_type'] = items[0].lower()
         pattern = r'(\S+)\:\/\/(\S+)'
@@ -383,6 +392,20 @@ def get_logging_info(job, args):
             logger.warning(f'protocol/url could not be extracted from {items}')
             info_dic['protocol'] = ''
             info_dic['url'] = ''
+    elif info:
+        try:
+            info_dic['logging_type'] = info[0][0]
+            info_dic['protocol'] = info[0][1]
+            info_dic['url'] = info[0][2]
+            info_dic['port'] = info[0][3]
+        except IndexError as exc:
+            logger.warning(f'exception caught: {exc}')
+            return {}
+        else:
+            # find the log file to tail
+            path = find_log_to_tail(job.debug_command, job.workdir, args, job.is_analysis())
+            logger.info(f'using {path} for real-time logging')
+            info_dic['logfiles'] = [path]
 
     if 'logfiles' not in info_dic:
         # Rubin
@@ -493,6 +516,7 @@ def run_realtimelog(queues, traces, args):  # noqa: C901
         info_dic = get_logging_info(job, args)
         if info_dic:
             args.use_realtime_logging = True
+            logger.debug("Going to get realtime_logger")
             realtime_logger = get_realtime_logger(args, info_dic, job.workdir, job.pilotsecrets)
         else:
             logger.debug('real-time logging not needed at this point')
@@ -534,7 +558,7 @@ def set_cpu_consumption_time(job):
 def perform_initial_payload_error_analysis(job, exit_code):
     """
     Perform an initial analysis of the payload.
-    Singularity errors are caught here.
+    Singularity/apptainer errors are caught here.
 
     :param job: job object.
     :param exit_code: exit code from payload execution.
@@ -544,7 +568,7 @@ def perform_initial_payload_error_analysis(job, exit_code):
     if exit_code != 0:
         logger.warning(f'main payload execution returned non-zero exit code: {exit_code}')
 
-    # look for singularity errors (the exit code can be zero in this case)
+    # look for singularity/apptainer errors (the exit code can be zero in this case)
     path = os.path.join(job.workdir, config.Payload.payloadstderr)
     if os.path.exists(path):
         stderr = read_file(path)
@@ -559,7 +583,7 @@ def perform_initial_payload_error_analysis(job, exit_code):
         msg = scan_for_memory_errors(job.subprocesses)
         if msg:
             job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(errors.PAYLOADOUTOFMEMORY, msg=msg)
-    elif exit_code != 0:
+    if exit_code != 0:
         msg = ""
 
         # are there any critical errors in the stdout?
