@@ -218,6 +218,23 @@ def tail(filename, nlines=10):
     return stdout
 
 
+def head(filename, count=20):
+    """
+    Return the first several line from the given file.
+
+    :param filename: file name (string).
+    :param count: number of lines (int).
+    :return: head lines (list).
+    """
+
+    ret = None
+    with open(filename, 'r') as _file:
+        lines = [_file.readline() for line in range(1, count + 1)]
+        ret = filter(len, lines)
+
+    return ret
+
+
 def grep(patterns, file_name):
     """
     Search for the patterns in the given list in a file.
@@ -931,7 +948,7 @@ def dump(path, cmd="cat"):
         logger.info(f"path {path} does not exist")
 
 
-def establish_logging(debug=True, nopilotlog=False, filename=config.Pilot.pilotlog, loglevel=0):
+def establish_logging(debug=True, nopilotlog=False, filename=config.Pilot.pilotlog, loglevel=0, redirectstdout=''):
     """
     Setup and establish logging.
 
@@ -940,13 +957,21 @@ def establish_logging(debug=True, nopilotlog=False, filename=config.Pilot.pilotl
       loglevel=0: '%(asctime)s | %(levelname)-8s | %(name)-32s | %(funcName)-25s | %(message)s'
       loglevel=1: 'ts=%(asctime)s level=%(levelname)-8s event=%(name)-32s.%(funcName)-25s msg="%(message)s"'
 
+    All stdout can be redirected to /dev/null (or to a file). Basically required in prompt processing, or there
+    will be too much stdout. If to a file, it is recommended to then also set an appropriate max pilot lifetime
+    to prevent it from creating too much stdout.
+
     :param debug: debug mode (Boolean),
     :param nopilotlog: True when pilot log is not known (Boolean).
     :param filename: name of log file (string).
     :param loglevel: selector for logging level (int).
+    :param redirectstdout: file name, or /dev/null (string).
     :return:
     """
 
+    if redirectstdout:
+        sys.stdout = open(redirectstdout, 'w')
+
     _logger = logging.getLogger('')
     _logger.handlers = []
     _logger.propagate = False
